pub mod types;

use tool_helper::Error;
use types::{MemoryMap, Section};

pub fn scan<F: FnMut() -> Option<String>>(mut next_line: F) -> Result<MemoryMap, Error> {
    const SECTION_IDENTIFIER:&'static str      = "Sections:";
    const SYMBOL_TABLE_IDENTIFIER:&'static str = "SYMBOL TABLE:";

    let mut memory_map = MemoryMap::default();

    while let Some(mut line) = next_line() {
        loop {
            // loop so we pick-up changes of line
            if line == SECTION_IDENTIFIER {
                line = scan_sections(&mut next_line, &mut memory_map.sections)?;
                continue;
            }

            if line == SYMBOL_TABLE_IDENTIFIER {
                // Process Symbol table here
            }

            break;
        }
    }

    // Print them off for now!
    for section in memory_map.sections {
        println!("Name: {}", section.name);
    }
    Err(Error::not_implemented("scan"))
}

fn scan_sections<F: FnMut() -> Option<String>>(next_line: &mut F, sections: &mut Vec<Section>) -> Result<String, Error> {
    while let Some(_) = next_line() {
        // We read every other line
        let line           = next_line().ok_or(Error::from_str("Failed skipping section line"))?;
        let mut split_line = line.split_whitespace();
        if let Ok(_) = split_line_radix(&mut split_line, 10, "") {
            let name = split_line.next().ok_or(Error::from_str("Failed reading Section Name"))?;
            let size = split_line_radix(&mut split_line, 16, "Section Size")?;
            let adr  = split_line_radix(&mut split_line, 16, "Section Address")?;

            sections.push(Section::new(name.to_string(), adr, size as usize));
        }

        else {
            return Ok(line);
        }
    }

    Err(Error::not_implemented("Failed obtaining complete section"))
}

fn split_line_radix<'a>(split_line: &mut std::str::SplitWhitespace<'a>, radix: u32, value_name: &str) -> Result<u64, Error> {
    match u64::from_str_radix(split_line.next().ok_or(Error::from_text(format!("Failed reading: {}", value_name)))?, radix) {
        Ok(value)  => Ok(value),
        Err(error) => Err(Error::from_text(format!("Converting value for {} failed with: {}", value_name, error)))
    }
}